@@ -1,5 +1,3 @@
-# $Id: TestServer.pm 23 2005-05-06 16:00:02Z martijn $
-
 package TestServer;
 
 use warnings;
@@ -7,26 +5,42 @@ use strict;
 
 use POE;
 use POE::Component::Server::TCP;
+use Socket qw(sockaddr_in);
 
 my %clients;
 my %servers;
+my $seq = 0;
 
 sub spawn {
   my ($class, $port) = @_;
 
-  my $alias = $servers{$port} = "server_$port";
+  my $alias = 'TestServer_' . ++$seq;
 
   POE::Component::Server::TCP->new(
     Alias              => $alias,
-    Port               => $port,
+    Port               => $port,   # Random one if 0.
     Address            => "127.0.0.1",
     ClientInput        => \&discard_client_input,
     ClientConnected    => \&register_client,
     ClientDisconnected => \&unregister_client,
+    Started            => sub {
+      # Switch $port to the port this server actually bound on.
+      my $listener_sockname = $_[HEAP]{listener}->getsockname();
+      if (defined $listener_sockname) {
+        ($port, undef) = sockaddr_in($listener_sockname);
+      }
+      else {
+        $port = undef;
+      }
+    },
     InlineStates       => {
       send_something   => \&internal_send_something,
     },
   );
+
+  $servers{$port} = $alias;
+
+  return $port;
 }
 
 sub register_client {
@@ -52,8 +66,8 @@ sub internal_send_something {
 }
 
 sub shutdown {
-  foreach my $session (values(%servers), keys(%clients)) {
-    $poe_kernel->post($session => "shutdown");
+  foreach my $alias (values(%servers), keys(%clients)) {
+    $poe_kernel->post($alias => "shutdown");
   }
 }
 
